// Header File
// Created 5/16/2005; 4:50:17 PM

#define WHITE					 0
#define BLACK					 1
#define NOBODY				-1
#define NONEXISTANT		-2

#define NOARROW				 0
#define ARROWOUT			 1
#define ARROWIN				 2

typedef struct
{
  int player;
  char dirFlags[8];
} EPEG;

typedef struct
{
  int startX,startY;
  int dir;
} ETURN;

typedef struct
{
  int curTurn;
  int xDim,yDim;
  int totalPegs;
  int x0,y0,x1,y1;
  EPEG *board;
  BOARDAREA theArea;
  long turnHistoryCount;
  ETURN *turnHistory;
} EBOARD;

/***********************************************************
player is 0 or 1 for a player or -1 for not owned

dirFlags are 8 characters for pieces from each direction.  It starts at straight up and goes clockwise

The dirFlags:

0: no arrow
1: arrow from current to stated dir
2: arrow from stated dir to current
*****************************************************************/

/*static long exGameLen=14;

static ETURN exampleGame[]={
	{0,0,3},
	{3,3,7},
	{1,1,3},
	{3,3,0},
	{0,0,4},
	{3,2,0},
	{0,1,4},
	{3,1,0},
	{0,2,4},
	{3,0,6},
	{0,3,2},
	{2,0,6},
	{1,3,2},
	{1,0,5}
};*/

static int dirChanges[8][2]={
	{0,-1},
	{1,-1},
	{1,0},
	{1,1},
	{0,1},
	{-1,1},
	{-1,0},
	{-1,-1}
};

EBOARD setupBoard(int xStart,int yStart,int xEnd,int yEnd,int xDim,int yDim)
{
  EBOARD ret={WHITE,xDim,yDim,xDim*yDim,xStart,yStart,xEnd,yEnd,(EPEG*)malloc(xDim*yDim*sizeof(EPEG)),boardArea(xStart,yStart,xEnd+1,yEnd+1,xDim*2+1,yDim*2+1),3,malloc(sizeof(ETURN)*3)};
  int i;
  ret.board[0].player=WHITE;
  ZeroMemory(ret.board[0].dirFlags,8);
  ret.board[0].dirFlags[0]=ARROWIN;
  ret.board[ret.totalPegs-1].player=BLACK;
  ZeroMemory(ret.board[ret.totalPegs-1].dirFlags,8);
  ret.board[ret.totalPegs-1].dirFlags[4]=ARROWIN;
  for (i=1;i<ret.totalPegs-1;i++)
  {
    ret.board[i].player=NOBODY;
    ZeroMemory(ret.board[i].dirFlags,8);
  }
  //this stores the game data in a turn history item
  ret.turnHistory->startX=xDim;
  ret.turnHistory->startY=yDim;
  ret.turnHistory[1].dir=0;			//this is where the layout goes; this is set to zero for compatibility with later versions
  return ret;
}

void addArrow(EBOARD *theBoard,int *x,int *y,int dir)
{
	theBoard->board[(*x)+(*y)*theBoard->xDim].dirFlags[dir]=ARROWOUT;
	*x+=dirChanges[dir][0];
	*y+=dirChanges[dir][1];
	if (*x<0||*y<0||*x>=theBoard->xDim||*y>=theBoard->yDim) return;
	theBoard->board[(*x)+(*y)*theBoard->xDim].dirFlags[(dir+4)%8]=ARROWIN;
}

void clearArrow(EBOARD *theBoard,int *x,int *y,int dir)
{
	theBoard->board[(*x)+(*y)*theBoard->xDim].dirFlags[dir]=NOARROW;
	*x+=dirChanges[dir][0];
	*y+=dirChanges[dir][1];
	if (*x<0||*y<0||*x>=theBoard->xDim||*y>=theBoard->yDim) return;
	theBoard->board[(*x)+(*y)*theBoard->xDim].dirFlags[(dir+4)%8]=NOARROW;
}

int calcPegMagnitude(EBOARD *theBoard,int x, int y)
{
	int i;
	int xbuf,ybuf;
	int playerbuf;
	int ret=0;
	char *flagptr=theBoard->board[x+y*theBoard->xDim].dirFlags;
	for (i=0;i<8;i++)
	{
		if (flagptr[i]==ARROWIN)
		{
			xbuf=x+dirChanges[i][0];
			ybuf=y+dirChanges[i][1];
			if (ybuf<0||xbuf<0)
			{
				ret--;
				continue;
			}
			if (ybuf>=theBoard->yDim||xbuf>=theBoard->xDim)
			{
				ret++;
				continue;
			}
			playerbuf=theBoard->board[xbuf+ybuf*theBoard->xDim].player;
			if (playerbuf<0)
			{
				clearArrow(theBoard,&x,&y,i);
				//state that there's an error
			}
			else if (playerbuf==WHITE) ret--;
			else if (playerbuf==BLACK) ret++;
		}
	}
	return ret;
}

int evaluatePeg(EBOARD *theBoard,int x,int y)
{
	int ret=calcPegMagnitude(theBoard,x,y);
	if (ret==0) return NOBODY;
	if (ret<0) return WHITE;
	return BLACK;
}

void setPeg(EBOARD *theBoard,int x,int y,int newPlayer)
{
	int *playerPtr=&(theBoard->board[x+y*theBoard->xDim].player);
	int xb,yb;
	int i;
	char *cfTemp;
	if (*playerPtr==newPlayer) return;
	if (*playerPtr>=0)
	{
		cfTemp=theBoard->board[x+y*theBoard->xDim].dirFlags;
		for (i=0;i<8;i++)
		{
			xb=x; yb=y;
			if (cfTemp[i]==ARROWOUT)
			{
				clearArrow(theBoard,&xb,&yb,i);
				setPeg(theBoard,xb,yb,evaluatePeg(theBoard,xb,yb));
			}
		}
	}
	*playerPtr=newPlayer;
}

int legalMove(EBOARD *theBoard,ETURN *theTurn)
{
  int x=theTurn->startX,y=theTurn->startY;
  int enemyMag;
  EPEG *temp=theBoard->board+(x+y*theBoard->xDim);
  if (temp->player!=theBoard->curTurn) return 0;
  if (temp->dirFlags[theTurn->dir]==ARROWOUT) return 0;
  x+=dirChanges[theTurn->dir][0];
  y+=dirChanges[theTurn->dir][1];
  if (x<0||y<0||x>=theBoard->xDim||y>=theBoard->yDim) return 0;
  temp=theBoard->board+(x+y*theBoard->xDim);
  if (temp->player==NONEXISTANT) return 0;
  if (temp->dirFlags[((theTurn->dir+4)%8)]==ARROWIN) return 0;
  if (temp->dirFlags[((theTurn->dir+4)%8)]==ARROWOUT)
  {
  	if (temp->player==theBoard->curTurn) return 0;
  	enemyMag=calcPegMagnitude(theBoard,x,y);
  	if (enemyMag<-1||enemyMag>1) return 0;
  }
  return 1;
}

int processTurn(EBOARD *theBoard,ETURN *theTurn)
{
	int x=theTurn->startX,y=theTurn->startY;
	if (!legalMove(theBoard,theTurn)) return 0;
	addArrow(theBoard,&x,&y,theTurn->dir);
	setPeg(theBoard,x,y,evaluatePeg(theBoard,x,y));
	return 1;
}

void changeTurn(EBOARD *theBoard)
{
	theBoard->curTurn=1-theBoard->curTurn;
}

void addTurnToHistory(EBOARD *theBoard,ETURN *theTurn)
{
	theBoard->turnHistoryCount++;
	theBoard->turnHistory=realloc(theBoard->turnHistory,theBoard->turnHistoryCount*sizeof(ETURN));
	memcpy(theBoard->turnHistory+theBoard->turnHistoryCount-1,theTurn,sizeof(ETURN));
}

void freeBoard(EBOARD *theBoard)
{
  free(theBoard->board);
  free(theBoard->turnHistory);
  theBoard->xDim=-1;
}

void getScores(EBOARD *theBoard,int *w,int *b)
{
  int i;
  int pbuffer;
  *w=0;
  *b=0;
  for (i=0;i<theBoard->totalPegs;i++)
  {
    pbuffer=theBoard->board[i].player;
    if (pbuffer==WHITE) (*w)++;
    else if (pbuffer==BLACK) (*b)++;
  }
}

int seekLegalMove(EBOARD *theBoard,int curX,int curY,int curDir,int seekWay)
{
	ETURN tbuffer;
	tbuffer.startX=curX;
	tbuffer.startY=curY;
	tbuffer.dir=curDir%8;
	while (!legalMove(theBoard,&tbuffer))
	{
		tbuffer.dir+=seekWay;
		if (tbuffer.dir>=8) tbuffer.dir-=8;
		if (tbuffer.dir<0) tbuffer.dir+=8;
		if (tbuffer.dir==curDir) return d_EXIT;
	}
	return tbuffer.dir;
}

#include "obsidioAI.h"